@@ -182,15 +182,17 @@ class ModelSettings(BaseSettings):
         default=True, description="Whether to print debug information."
     )
 
-    @model_validator(mode="before")  # pre=True to ensure this runs before any other validation
+    @model_validator(
+        mode="before"
+    )  # pre=True to ensure this runs before any other validation
     def set_dynamic_defaults(self) -> Self:
         # If n_threads or n_threads_batch is -1, set it to multiprocessing.cpu_count()
         cpu_count = multiprocessing.cpu_count()
         values = cast(Dict[str, int], self)
-        if values.get('n_threads', 0) == -1:
-            values['n_threads'] = cpu_count
-        if values.get('n_threads_batch', 0) == -1:
-            values['n_threads_batch'] = cpu_count
+        if values.get("n_threads", 0) == -1:
+            values["n_threads"] = cpu_count
+        if values.get("n_threads_batch", 0) == -1:
+            values["n_threads_batch"] = cpu_count
         return self
 
 
